@@ -28,7 +28,7 @@ pub struct Data<'a> {
     pub step_count: &'a mut i32,
 
     pub step_vars: HashMap<String, Literal>,
-    pub previous_info: PreviousInfo,
+    pub previous_info: Option<PreviousInfo>,
     pub custom_component: &'a serde_json::Map<String, serde_json::Value>,
     pub native_component: &'a serde_json::Map<String, serde_json::Value>,
 }
@@ -65,7 +65,7 @@ impl<'a> Data<'a> {
         loop_index: usize,
         step_count: &'a mut i32,
         step_vars: HashMap<String, Literal>,
-        previous_info: PreviousInfo,
+        previous_info: Option<PreviousInfo>,
         custom_component: &'a serde_json::Map<String, serde_json::Value>,
         native_component: &'a serde_json::Map<String, serde_json::Value>,
     ) -> Self {
@@ -114,6 +114,14 @@ impl<'a> Data<'a> {
             self.native_component.clone(),
         )
     }
+
+    // get permanent and temporary memories in a single hashmap
+    pub fn get_all_memories(&self) -> HashMap<String, Literal> {
+        let remember_memory = self.context.current.clone();
+        let step_memory = self.step_vars.clone();
+
+        remember_memory.into_iter().chain(step_memory).collect()
+    }
 }
 
 pub fn init_child_context(data: &Data) -> Context {
