@@ -599,12 +599,12 @@ impl PrimitiveArray {
                     )?
                     .to_owned();
 
-                    if int_start.is_negative() {
+                    if int_start < 0 {
                         int_start = len as i64 + int_start;
                     }
 
                     let start = match int_start {
-                        value if value.is_positive() && (value as usize) < len => value as usize,
+                        value if value >= 0 && (value as usize) < len => value as usize,
                         _ => {
                             return Err(gen_error_info(
                                 Position::new(interval, &data.context.flow),
@@ -642,7 +642,7 @@ impl PrimitiveArray {
                     )?
                     .to_owned();
 
-                    if int_start.is_negative() {
+                    if int_start < 0 {
                         int_start = len as i64 + int_start;
                     }
 
@@ -658,8 +658,8 @@ impl PrimitiveArray {
 
                     let (start, end) = match (int_start, int_end) {
                         (start, end)
-                            if start.is_positive()
-                                && end.is_positive()
+                            if int_start >= 0
+                                && end >= 0
                                 && (start as usize) < len
                                 && (end as usize) <= len =>
                         {
