@@ -12,7 +12,7 @@ use crate::data::position::Position;
 use crate::data::{ast::*, Data, Hold, IndexInfo, Literal, MessageData, MSG};
 use crate::error_format::*;
 use crate::interpreter::{
-    ast_interpreter::{for_loop, match_actions, solve_if_statement},
+    ast_interpreter::{for_loop, while_loop, match_actions, solve_if_statement},
     variable_handler::{expr_to_literal, interval::interval_from_expr},
 };
 use crate::parser::ExitCondition;
@@ -91,6 +91,7 @@ pub fn interpret_scope(
                     step_vars_to_json(map),
                     data.context.step.clone(),
                     data.context.flow.clone(),
+                    data.previous_info.clone(),
                 );
 
                 message_data.hold = Some(hold.to_owned());
@@ -123,12 +124,19 @@ pub fn interpret_scope(
                     &sender,
                 )?
             }
+            Expr::WhileExpr(expr, block, range) => {
+                message_data = while_loop(
+                    expr,
+                    block,
+                    range,
+                    message_data,
+                    data,
+                    &sender,
+                )?
+            }
             e => {
                 return Err(gen_error_info(
-                    Position::new(
-                        interval_from_expr(e),
-                        &data.context.flow,
-                    ),
+                    Position::new(interval_from_expr(e), &data.context.flow),
                     ERROR_START_INSTRUCTIONS.to_owned(),
                 ));
             }
