@@ -1,10 +1,11 @@
+use crate::data::data::PreviousInfo;
 use crate::data::position::Position;
 use crate::data::{
     ast::*,
     data::Data,
     literal::ContentType,
     message::*,
-    primitive::{closure::capture_variables, PrimitiveNull},
+    primitive::{closure::capture_variables, PrimitiveNull, PrimitiveString},
     Literal, Memory, MemoryType, MessageData, MSG
 };
 use crate::error_format::*;
@@ -13,6 +14,7 @@ use crate::interpreter::variable_handler::{
     search_goto_var_memory, forget_memories::{forget_scope_memories, remove_message_data_memories}
 };
 use crate::parser::ExitCondition;
+use std::collections::HashMap;
 use std::sync::mpsc;
 
 fn get_var_info<'a>(
@@ -85,7 +87,7 @@ pub fn match_actions(
             expr_to_literal(arg, false, None, data, &mut msg_data, sender)?;
             Ok(msg_data)
         }
-        ObjectType::Do(DoType::Update(old, new)) => {
+        ObjectType::Do(DoType::Update(assign_type, old, new)) => {
             // ######################
             // TODO:
             // create a temporary scope, this is necessary in order to bypass de borrow checker
@@ -119,12 +121,51 @@ pub fn match_actions(
                 &tmp_native_component,
             );
             // #####################
+
             let mut new_value = expr_to_literal(new, false, None, data, &mut msg_data, sender)?;
 
             // only for closure capture the step variables
-            capture_variables(&mut &mut new_value, data.step_vars.clone(), &data.context.flow);
+            let memory: HashMap<String, Literal> = data.get_all_memories();
+            capture_variables(&mut &mut new_value, memory, &data.context.flow);
 
             let (lit, name, mem_type, path) = get_var_info(old, None, data, &mut msg_data, sender)?;
+            match assign_type {
+                AssignType::AdditionAssignment => {
+                    let primitive = lit.primitive.clone() + new_value.primitive;
+
+                    match primitive {
+                        Ok(primitive) => {
+                            new_value = Literal {
+                                content_type: new_value.content_type,
+                                interval: new_value.interval,
+                                primitive
+                            };
+                        }
+                        Err(err) => {
+                            new_value = PrimitiveString::get_literal(&err, lit.interval)
+                        }
+                    }
+                }
+                AssignType::SubtractionAssignment => {
+                    let primitive = lit.primitive.clone() - new_value.primitive;
+
+                    match primitive {
+                        Ok(primitive) => {
+                            new_value = Literal {
+                                content_type: new_value.content_type,
+                                interval: new_value.interval,
+                                primitive
+                            };
+                        }
+                        Err(err) => {
+                            new_value = PrimitiveString::get_literal(&err, lit.interval)
+                        }
+                    }
+                }
+                _ => {}
+            };
+            
+
             exec_path_actions(
                 lit,
                 false,
@@ -164,7 +205,15 @@ pub fn match_actions(
             );
 
             // previous flow/step
-            data.previous_info.goto(data.context.flow.clone(), data.context.step.clone());
+            match data.previous_info {
+                Some(ref mut previous_info) => {
+                    previous_info.goto(data.context.flow.clone(), data.context.step.clone());
+                }
+                None => {
+                    data.previous_info = Some(PreviousInfo::new(data.context.flow.clone(), data.context.step.clone()))
+                }
+            }
+
             // current flow/step
             data.context.step = step.to_string();
             msg_data.exit_condition = Some(ExitCondition::Goto);
@@ -187,7 +236,14 @@ pub fn match_actions(
             );
 
             // previous flow/step
-            data.previous_info.goto(data.context.flow.clone(), data.context.step.clone());
+            match data.previous_info {
+                Some(ref mut previous_info) => {
+                    previous_info.goto(data.context.flow.clone(), data.context.step.clone());
+                }
+                None => {
+                    data.previous_info = Some(PreviousInfo::new(data.context.flow.clone(), data.context.step.clone()))
+                }
+            }
             // current flow/step
             data.context.step = "start".to_string();
             data.context.flow = flow.to_string();
@@ -224,7 +280,15 @@ pub fn match_actions(
             );
 
             // previous flow/step
-            data.previous_info.goto(data.context.flow.clone(), data.context.step.clone());
+            match data.previous_info {
+                Some(ref mut previous_info) => {
+                    previous_info.goto(data.context.flow.clone(), data.context.step.clone());
+                }
+                None => {
+                    data.previous_info = Some(PreviousInfo::new(data.context.flow.clone(), data.context.step.clone()))
+                }
+            }
+
             // current flow/step
             data.context.flow = flow.to_string();
             data.context.step = step.to_string();
@@ -235,30 +299,36 @@ pub fn match_actions(
             let flow_opt;
             let mut step_opt = None;
 
-            match previous_type {
-                PreviousType::Flow(_interval) => {
-                    let tmp_f = data.previous_info.flow.clone();
+            match (previous_type, &mut data.previous_info) {
+                (PreviousType::Flow(_interval), Some(ref mut previous_info)) => {
+                    let tmp_f = previous_info.flow.clone();
                     flow_opt = Some(tmp_f.clone());
 
-                    data.previous_info.flow = data.context.flow.clone();
-                    data.previous_info.step_at_flow = (data.context.step.clone(), data.context.flow.clone());
+                    previous_info.flow = data.context.flow.clone();
+                    previous_info.step_at_flow = (data.context.step.clone(), data.context.flow.clone());
 
                     data.context.flow = tmp_f;
                     data.context.step = "start".to_string();
                 },
-                PreviousType::Step(_interval) => {
-                    let (tmp_s, tmp_f) = data.previous_info.step_at_flow.clone();
+                (PreviousType::Step(_interval), Some(ref mut previous_info)) => {
+                    let (tmp_s, tmp_f) = previous_info.step_at_flow.clone();
                     flow_opt = Some(tmp_f.clone());
                     step_opt = Some(tmp_s.clone());
 
                     if data.context.flow != tmp_f {
-                        data.previous_info.flow = tmp_f.clone();
+                        previous_info.flow = tmp_f.clone();
                     }
-                    data.previous_info.step_at_flow = (data.context.step.clone(), data.context.flow.clone());
+                    previous_info.step_at_flow = (data.context.step.clone(), data.context.flow.clone());
 
                     data.context.flow = tmp_f;
                     data.context.step = tmp_s;
                 }
+                (_, None) => {
+                    flow_opt = None;
+                    step_opt = Some("end".to_owned());
+
+                    data.context.step = "end".to_string();
+                }
             }
 
             msg_data.exit_condition = Some(ExitCondition::Goto);
@@ -278,7 +348,8 @@ pub fn match_actions(
                 expr_to_literal(variable, false, None, data, &mut msg_data, sender)?;
 
             // only for closure capture the step variables
-            capture_variables(&mut &mut new_value, data.step_vars.clone(), &data.context.flow);
+            let memory: HashMap<String, Literal> = data.get_all_memories();
+            capture_variables(&mut &mut new_value, memory, &data.context.flow);
 
             msg_data.add_to_memory(&name.ident, new_value.clone());
 
